import importlib as imp
from utils import*
from crystals import Crystal
from scattering import structure_factor as sf;imp.reload(sf)

crys = Crystal.from_database('Si')
pattern = np.array([np.hstack([a.coords_fractional,0*a.atomic_number]) for a in crys.atoms] )
lat_vec = np.array(crys.reciprocal_vectors)
hkl,Fhkl=sf.structure_factor3D(pattern,lat_vec,hkl=None,hklMax=2,sym=0,v='')
h,k,l = hkl
I = np.abs(Fhkl)**2

id0 = ~((h%2==0) & (k%2==0) & (l%2==0)) & ~((h%2==1) & (k%2==1) &(l%2==1))

print(np.abs(I[id0]))                   #should be 0
print(np.abs(I[~id0 & ((h+k+l)%4==0)])) #should be 64
print(np.abs(I[~id0 & ((h+k+l)%2==1)])) #should be 32
print(np.abs(I[~id0 & ((h+k+l)%4==2)])) #should be 0



from utils import*
from scattering import structure_factor as sf;imp.reload(sf)

def solve_Bloch(k0,hkl,Gs,Fhkl,v=0):
    ''' Solves fast electron wave equation with Bloch wave system
    k0   : incident wave vector amplitude
    idx  : Nx2 or Nx3 - miller indices of each beam
    Gs   : Nx2 or Nx3 - reciprocal lattice beams
    Vhkl : 2Nx+1 x 2Ny+1 - 3D Potential components (Volts)
    '''
    Ug = cst.Vg2Ug(Fhkl,k0)
    N,dim = Gs.shape                    # Number of beams and dimensions 2D/3D
    z0 = np.zeros((dim)); z0[-1] = 1    # beam direction
    K0 = k0*z0                          # Incident beam
    # V0 index
    Nx,Ny = Fhkl.shape                    # nb Fhkl components in x and y directions
    i_hkl0 = (np.array(Fhkl.shape)-1)/2   # Such that Fhkl[i_hkl0] = V0
    i_hkl0 = np.array(i_hkl0,dtype=int)
    #normalized excitation errors
    Sg = (k0**2-np.linalg.norm(K0-Gs,axis=1)**2)/(2*k0)         #;print(Sg)

    if v:print(green+'\t ...Assembling matrix with %d x %dD beams ...' %(N,dim) +black)
    # H = np.diag(Sg+0J);
    H = np.zeros((N,N),dtype=complex)
    for iG,G in enumerate(hkl) :
        #Vg is accessed as Fhkl[i,j] where i,j = G-hkl
        V_iG = np.array([Fhkl[tuple(hkl_p+i_hkl0)] for hkl_p in G-hkl]) #;print(V_iG.shape)
        V_iG = V_iG/(2*k0) #;print(iG,V_iG)
        H[iG,iG] = Sg[iG]   #diagonal terms as excitation errors
        H[iG,:] += V_iG     #off diagonal terms as potential
    if v:print(H.real)
    gammaj,CjG = np.linalg.eigh(H) #;print(red+'Ek',lk,black);print(wk)
    return gammaj,CjG


###############################################################################
#### misc
###############################################################################
def get_pattern(name,N=1,opt='',**kwargs):
    # crys = Crystal.from_database(name)
    # lat_vec  = crys.reciprocal_vectors
    # pattern  = np.array([list(a.coords_fractional)+[a.atomic_number] for a in crys.atoms])
    pptype,ax,by,cz,angle = 'p1',2,2,2,90
    pattern = np.array([[1,1,1,1]])
    lat_vec = 2*np.pi*np.diag([1/ax,1/by,1/cz])
    hkl,Fhkl = sf.structure_factor3D(pattern,lat_vec,hklMax=2*N,sym=1)
    if opt:sFact.plot_structure3D(hkl,Fhkl,opt=opt,**kwargs)
    a,b,c = lat_vec
    hkl = np.array([i[N:3*N+1,N:3*N+1,N:3*N+1].flatten() for i in hkl])
    #hkl = hkl[0].flatten(),hkl[1].flatten(),hkl[2].flatten()])
    h,k,l = hkl
    kx = a[0]*h + b[0]*k + c[0]*l
    ky = a[1]*h + b[1]*k + c[1]*l
    kz = a[2]*h + b[2]*k + c[2]*l
    Gs = np.array([kx,ky,kz]).T/(2*np.pi)
    Vg = Fhkl #.flatten()
    return hkl.T,Gs,Vg/(ax*by*cz)


def get_pattern2D(N,s,**kwargs):
    ''' Setup for Bloch wave simulation
        - N : number of beams such that Vg.shape= 2N+1 x 2N+1
        - s : np.s_[...] - slices of Vg beams for the G beams to perform blochwave calculation
    '''
    pptype,ax,cz,angle = 'p1',2,2,90
    pattern = np.array([[1,1,1]])
    lat_vec = 2*np.pi*np.diag([1/ax,1/cz])
    hl,Fhl = sf.structure_factor2D(pattern,lat_vec,hkMax=2*N,sym=1,eps=0.01)   #;print(hl)
    a,c = lat_vec
    h,l = hl[0][s],hl[1][s]
    kx = a[0]*h + c[0]*l
    kz = a[-1]*h + c[-1]*l
    Gs = np.array([kx,kz]).T/(2*np.pi)
    # Fhl = hl[0]+hl[1] # Was used to test assembling
    return np.array([h,l]).T,Gs,Fhl/(ax*cz)



########################################################################
#def : test
########################################################################
def test_Ug():
    #sanity check on units here
    k0 = 5.008343065817388      #A-1
    keV = cst.lam2keV(1/k0)     #keV
    sig = cst.keV2sigma(keV)    #kVA
    Vg = 1                      #Volts
    xig = np.pi/(sig*Vg/1000)   #A
    print(xig,k0/cst.Vg2Ug(Vg,k0))


if __name__ == '__main__':
    test_Ug()
    # print('bloch wave')



import importlib as imp
from utils import*                  ;imp.reload(dsp)
import multislice.multi_2D as ms    ;imp.reload(ms)
import wallpp.plane_group as pg     ;imp.reload(pg)
plt.close('all')
figpath='../../multislice/docs_fig/multi2D/'

ax,bz = 2,2
pattern = np.array([[1,1,1]])
ndeg = 2**11
nzx = 32

# def compare_patterns(mp1,mp2):
#     mp1.

def apply_padding(p1,npad,nx,name='pad',**kwargs):
    x0,z0 = np.linspace(-npad*ax,(nx+npad)*ax,ndeg),np.linspace(0,bz,nzx)
    x,z = np.meshgrid(x0,z0)
    Xv = np.stack([x.flatten(),z.flatten()]).T
    f = np.sum(np.array([pg.fv(Xv,X0,int(Za)) for X0,Za in zip(p1.Xa,p1.fa)]),axis=0)
    f = np.reshape(f,(nzx,ndeg))
    pattern = [x0,z0,f]

    mp0 = ms.Multi2D(pattern,ax,bz,keV=200,
            Nx=1,dz=bz,nz=0,ppopt='',
            iZs=1,iZv=10,eps=1)
    # mp0.V_show(opt='p')
    mp0.Tz_show(iSz=slice(0,None,1),Vopt='VT',lw=2,name=figpath+name+'Tz.svg',**kwargs)
    mp0.propagate(201)
    # mp0.Xxz_show()#iZs=1,iXs=2)
    # mp0.Bz_show(iBs=np.arange(1,20,4),tol=1e-2,cmap='jet',lw=2,name=figpath+name+'Bz.svg',**kwargs)
    mp0.Qz_show(iZs=50,lw=2,opts='Nl',name=figpath+name+'Qz.svg',xylims=['x',-3,3],**kwargs)
    return mp0

nx=1
# p1 = pg.Wallpaper(pp_type='p1',a=ax,b=bz,angle=90,pattern=pattern,ndeg=0,nh=nx,nk=1)
# apply_padding(p1,npad=0,nx=nx,name='pad0_nx1',opt='p')
# apply_padding(p1,npad=1,nx=nx,name='pad1_nx1',opt='p')
# apply_padding(p1,npad=2,nx=nx,name='pad2_nx1',opt='p')
# apply_padding(p1,npad=5,nx=nx,name='pad3_nx1',opt='p')

# nx=2
# p1 = pg.Wallpaper(pp_type='p1',a=ax,b=bz,angle=90,pattern=pattern,ndeg=0,nh=2,nk=1)
# apply_padding(p1,npad=0 ,nx=nx,name='pad0_nx2',opt='sp')
# apply_padding(p1,npad=5 ,nx=nx,name='pad1_nx2',opt='sp')
# apply_padding(p1,npad=10,nx=nx,name='pad2_nx2',opt='sp')
# apply_padding(p1,npad=20,nx=nx,name='pad3_nx2',opt='sp')

npad=20
# p1a = pg.Wallpaper(pp_type='p1',a=ax,b=bz,angle=90,pattern=pattern,ndeg=0,nh=1,nk=1)
# p1b = pg.Wallpaper(pp_type='p1',a=ax,b=bz,angle=90,pattern=pattern,ndeg=0,nh=2,nk=1)
# p1c = pg.Wallpaper(pp_type='p1',a=ax,b=bz,angle=90,pattern=pattern,ndeg=0,nh=5,nk=1)
p1d = pg.Wallpaper(pp_type='p1',a=ax,b=bz,angle=90,pattern=pattern,ndeg=0,nh=10,nk=1)
# msa = apply_padding(p1a,npad=npad,nx=1 ,name='padnx1',opt='p')
# msb = apply_padding(p1b,npad=npad,nx=2 ,name='padnx2',opt='p')
# msc = apply_padding(p1c,npad=npad,nx=5 ,name='padnx3',opt='p')
msd = apply_padding(p1d,npad=npad,nx=10,name='padnx4',opt='p')



from utils import*
from math import factorial as fact
path='../multislice/docs_fig/multi2D/Pcoh_kin_dual_dyn'
plt.close('all')
topt = 1        #number of multiple scattering to show
rho   = 0.106   #atoms/A^3
sig_e = np.array([0.001,0.0025,0.005]) #A^2
le_s  = 1/(sig_e*rho) #A

z = np.linspace(0,10000,1000) #A

cs=dsp.getCs('Reds',topt+2)
l_e  = le_s[1]
y = [1/fact(m)*(z/l_e)**m*np.exp(-z/l_e) for m in range(topt+1)]
plts = [ [z/10,y[m],cs[m],'$%d$' %m] for m in range(topt+1)]
plts+= [[z/10,1-np.array(y).sum(axis=0),'k--','']]
dsp.stddisp(plts,lw=2,labs=[r'$z(nm)$','P(z)'],#title=tle,
    name=path+'%d.svg' %0, opt='p',fonts={'leg':20,'title':20})


# for i,le in zip(range(le_s.size),le_s):
#     Pcoh = np.exp(-z/le)
#     Pkin = z/le*np.exp(-z/le)
#     Pdual = 0.5*(z/le)**2*np.exp(-z/le)
#     if topt:
#         Ptri = 1/6*(z/le)**3*np.exp(-z/le)
#         Pdyn = 1-np.exp(-z/le)*(1+z/le+0.5*(z/le)**2+(z/le)**3/6)
#         S = Pcoh+Pkin+Pdual+Ptri+Pdyn
#     else:
#         Pdyn = 1-np.exp(-z/le)*(1+z/le+0.5*(z/le)**2)
#         S = Pcoh+Pkin+Pdual+Pdyn
#
#     tle = r'$\sigma_e=%.3f\AA^2$, $l_e$=%.d nm' %(sig_e[i],int(le/10))
#     cs=dsp.getCs('jet',4)
#     plts = [[z/10,Pcoh,cs[0],'$coh$'],[z/10,Pkin,cs[1],'$kin$']]
#     plts+= [[z/10,Pdual,cs[2],'$double$']]
#     if topt: plts+= [[z/10,Ptri ,cs[3],'$tri$']]
#     plts+= [[z/10,Pdyn,'k','$dyn$']]#,[z/10,S,'k--']]
#     dsp.stddisp(plts,lw=2,labs=[r'$z(nm)$','P(z)'],#title=tle,
#         name=path+'%d.svg' %i, opt='p',fonts={'leg':20,'title':20})



import importlib as imp
import pickle
from scattering.structure_factor import structure_factor3D
import multislice.multislice as mupy; imp.reload(mupy)
import multislice.postprocess as pp ; imp.reload(pp)
import multislice.mupy_utils as mut ; imp.reload(mut)
from crystals import Crystal
from utils import*
plt.close('all')
path = '../dat/biotin/'
figpath = '../docs_fig/'
file = path+'biotin.cif'
opts = 'T '  #E(Ewald) T(tilts) B(Beams)F(structFact)


crys = Crystal.from_cif(file)
lat_params = crys.lattice_parameters[:3]
a1,a2,a3 = lat_params
b12 = np.sqrt(1/a1**2+1/a2**2)

fs = {'leg':30,'lab':40,'tick':25}
if 'E' in opts:
    #Ewald configuration [010]
    mut.ewald_sphere(lat_params,lam=cst.eV2mum(12000)*1e4,tmax=60,T=20.,nx=5 ,ny=20,opt='p',name=figpath+'E_MX.png',xylims=[-1,1.01,-0.1,2.01],xyTicks=1,legLoc='upper right',fonts=fs)
    mut.ewald_sphere(lat_params,lam=cst.keV2lam(200)     ,tmax=7 ,T=0.2,nx=20,ny=10,opt='p',name=figpath+'E_ED.png',xylims=[-3,3.01,-0.1,1.01],xyTicks=1,legLoc='upper right',fonts=fs)

if 'B' in opts:
    df = pd.read_pickle(path+'df0.pkl')
    p1 = pp.load_multi_obj(path+'biotin_m0_autoslic.pkl')
    p1.datpath = p1.datpath.replace('ronan','tarik')
    p1.get_beams(bOpt='na')

    #plot
    iBs = ['(64,0)','(0,32)','(0,64)','(32,32)','(64,64)']
    fig,ax = p1.beam_vs_thickness(iBs=iBs,cm='jet',
        fonts=fs,opt='ps',name=figpath+'biotin_m0_beams.png')
    if 'F' in opts:
        # structure factor
        lat_vec = crys.reciprocal_vectors
        pattern = np.array([ np.hstack([a.coords_cartesian,a.atomic_number]) for a in crys.atoms])
        hkl,F3d = structure_factor3D(pattern,lat_vec,hkl=None,hklMax=3,sym=0)
        #excitation error
        K = 1/cst.keV2lam(200)
        k = [2/a1,2/a2, b12, 2*b12]
        ti = np.arcsin(k/K)
        xi = K*(1-np.cos(ti))
        z = np.linspace(0,2000,2000)
        Fibs = F3d[[2,0,0,1,2],[0,1,2,1,2],0]
        Fi = [Fibs[i]*np.sin(np.pi*z*zeta_i)/(np.pi*zeta_i) for i,zeta_i in enumerate(xi)]
        Ii = np.abs(Fi)**2
        #plot kinematic
        cs = dsp.getCs('jet',len(iBs))
        plts = [[z,I/1e8,[cs[i],'--'],''] for i,I in enumerate(Ii)]
        dsp.pltPlots(ax,plts,2)
        fig.show()



if 'T' in opts:
    # df = pp.update_df_info(path+'tilts/dfm.pkl',hostpath='/data3/lii26466/multislice/biotin/tilts/')#; print(df)
    # for i,f in enumerate(df.index):
    #     p1 = pp.load_multi_obj(path+'tilts/'+f)
    #     e=p1.ssh_get('badb',file='beamstxt',hostpath='/data3/lii26466/multislice/biotin/tilts/')#;print(e)
        # p1.get_beams(bOpt='fa')

    df = pd.read_pickle(path+'tilts/df.pkl')
    tilts = np.rad2deg(np.array([t[1] for t in df.tilt.values])/    1000)
    iBs = ['(4,0)','(0,2)','(0,4)','(2,2)','(4,4)']
    It = np.zeros((tilts.size,len(iBs)))
    for i,f in enumerate(df.index):
        p1 = pp.load_multi_obj(path+'tilts/'+f)
        p1.datpath = p1.datpath.replace('ronan','tarik')
        # p1.get_beams(bOpt='fa')
        It[i,:] = p1.beam_vs_thickness(iBs=iBs,bOpt='o')[-1][:,-1]
    cs = dsp.getCs('jet',len(iBs))
    plts = [[tilts,It[:,i],[cs[i],'-o'],iB] for i,iB in enumerate(iBs)]
    dsp.stddisp(plts,lw=2,labs=[r'$\theta(deg)$','$I$'],fonts=fs,name=figpath+'biotin_tilts_rocking.png',opt='ps')
    # p1.get_beams()
    # p1.beam_vs_thickness() #iBs=['(2,17)','(8,34)'],bOpt='f')
    # p1.save_pattern()
    # p1.pattern(Iopt='Isnlg',tol=1e-4,rings=[0,0.25,0.5,1],caxis=[-6.2,0],
    #     pOpt='ptX',xylims=[-1,1,-1,1],cmap='binary',imOpt='ch',axPos=[0.2,0.13,0.75,0.75])
    # h,k,I=p1.pattern(Iopt='Isnlg',tol=1e-4,out=1)
    # np.save('biotin_m1.npy',I)



import importlib as imp
import os,glob, numpy as np, pandas as pd
from subprocess import Popen,PIPE
from crystals import Crystal
from utils import handler3D as h3D          ;imp.reload(h3D)
from utils import displayStandards as dsp   ;imp.reload(dsp)
from . import mupy_utils as mut             ;imp.reload(mut)
from .rotating_crystal import get_crystal_rotation

class Pets:
    def __init__(self,pts_file,cif_file=None,gen=False):
        '''
        - pts_file : str - full path to .pts file
        - cif_file : str - full path to .cif file (automatically found if None)
        - gen      : bool - regenerate if True
        '''
        self.name  = os.path.basename(pts_file).split('.pts')[0] #;print(self.name)
        self.path  = os.path.dirname(pts_file)
        self.out   = self.path+'/dat/'

        self.cif_file   = mut._get_cif_file(self.path,cif_file)
        self.crys       = Crystal.from_cif(self.cif_file)
        self.lat_vec    = np.array(self.crys.lattice_vectors)
        self.lat_vec1   = np.array(self.crys.reciprocal_vectors)/(2*np.pi)
        self.lat_params = self.crys.lattice_parameters

        gen |= not os.path.exists(self.out)
        # if os.path.exists(self.out):gen |= not os.path.exists(self.out+'rpl.txt')
        if gen:self.convert_pets()
        self.load_all()
        self.nFrames = self.uvw.shape[0]


    def convert_pets(self):

        cmd = "%s/convert_pets.sh %s %s" %(os.path.dirname(__file__),self.path,self.name)
        p = Popen(cmd,shell=True,stdout=PIPE,stderr=PIPE);p.wait();o,e=p.communicate()
        print(o.decode()) #; print(e.decode())

        A = np.loadtxt(self.out+'UB.txt')
        np.save(self.out+'UB.npy',np.reshape(A,(3,3)))
        self.load_all()

    def load_all(self):
        lam,omega,aper = np.loadtxt(self.out+'pts.txt')
        self.omega = omega
        self.aper  = aper
        self.lam   = lam
        self.K0    = 1/self.lam

        self.frames = pd.read_csv(self.out+'iml.txt',sep=',',names=['name','alpha','beta','domega','scale','calibration','ellipA','ellipP','used'])
        self.rpl = pd.read_csv(self.out+'rpl.txt',sep=',',names=['x','y','z','I','i','px','py','rpx','rpy','alpha','Im','F'])
        self.cor = pd.read_csv(self.out+'cor.txt',sep=',',names=['x','y','z','I','i','px','py','rpx','rpy','alpha','Im','F'])
        self.cen = pd.read_csv(self.out+'cenloc.txt',sep=',',names=['px','py','m','n'])
        self.xyz = pd.read_csv(self.out+'xyz.txt',sep=',',names=['x','y','z','I','u0','px','py','F','alpha','Im','u1'])
        self.hkl = pd.read_csv(self.out+'hkl.txt',sep=',',names=['h','k','l','I','i','F','L'])
        self.cif = pd.read_csv(self.out+'cif.txt',sep=',',names=['id','u','v','w','prec','alpha','beta','omega','scale'])
        self.HKL = pd.read_csv(self.out+'HKL.txt',sep=',',names=['h','k','l','I','sig','F'])
        self.A   = np.load(self.out+'UB.npy')
        self.invA = np.linalg.inv(self.A)

        uvw   = self.cif[['u','v','w']].values
        beams = self.lat_vec.T.dot(uvw.T).T
        self.uvw   = uvw/np.linalg.norm(uvw,axis=1)[:,None]
        self.beams = self.K0*beams/np.linalg.norm(beams,axis=1)[:,None]
        self.XYZ   = self.xyz[['x','y','z']].values.T

        rpl_hkl = self.invA.dot(self.rpl[['x','y','z']].values.T)
        self.rpl[['hx','kx','lx']]  = rpl_hkl.T
        self.rpl[['h','k','l']]     = np.array(np.round(rpl_hkl),dtype=int).T

        cx,cy = self.cen[['px','py']].iloc[self.rpl.F-1].values.T
        px,py = self.rpl[['px','py']].values.T
        qxqy  = self.aper*(np.vstack([px,-py]).T-np.vstack([cx,-cy]).T)
        self.rpl[['qx','qy']] = qxqy

    ###########################################################################
    #### get
    ###########################################################################
    def get_lattice(self,Nmax=5):
        return mut.get_lattice(self.lat_vec1,Nmax)

    def get_beam_dir(self,frame):
        return self.uvw[frame-1]

    def get_beam(self,frame):
        return self.beams[frame-1]

    def get_ewald(self,frame,nts=100,nps=200):
        return mut.get_ewald(self.get_beam(frame),nts,nps)

    def get_excitation_errors(self,frame,Nmax=5,Smax=0.02):
        return mut.get_excitation_errors(self.get_beam(frame),self.lat_vec1,Nmax=Nmax,Smax=Smax)

    def get_kin(self,frame,thick,Nmax=5,Smax=0.02,e0=[1,0,0],rot=0,Imag=10):
        K = self.get_beam(frame)#;print(K)
        df = mut.get_kinematic_intensities(self.cif_file,K,thick,Nmax=Nmax,Smax=Smax)
        qxyz = df[['qx','qy','qz']].values
        I = df.I.values*Imag

        px,py = mut.project_beams(K,qxyz,e0)
        if rot:
            ct,st = np.cos(np.deg2rad(rot)),np.sin(np.deg2rad(rot))
            px,py = ct*px-st*py,st*px+ct*py
        hkl = df[['h','k','l']].values.T
        return px,py,I,hkl

    ###########################################################################
    #### display
    ###########################################################################
    def show_ewald_sphere(self,frame=None,Smax=0.01,Nmax=10,h3d=0,
        nts=100,nps=200,**kwargs):
        K = self.get_beam(frame)
        Kx,Ky,Kz = K

        (h,k,l),(qx,qy,qz) = self.get_lattice(Nmax)
        x,y,z = self.get_ewald(frame,nts,nps)
        df    = self.get_excitation_errors(frame,Smax=Smax,Nmax=Nmax)

        print('small excitation error beams',uvw)
        if frame:print('frame %d:' %frame)
        print(df[['h','k','l']])

        scat = ([qx,qy,qz,5,'b','o'],[0,0,0,'r','s'])
        scat+= ([df.qx,df.qy,df.qz,50,'g','s'],)
        surf = [x,y,z,{'alpha':0.4,'color':'r'}]
        plts = [[0,Kx],[0,Ky],[0,Kz],'r']
        # scat+=([x0,y0,z0])
        fig,ax = dsp.stddisp(plts,scat=scat,surfs=[surf],lw=3,
            labs=['x','y','z'],rc='3d',**kwargs)
        if h3d:h3d = h3D.handler_3d(fig,persp=False)

    def show_uvw(self):
        uvw = self.uvw
        ez =  np.cross(uvw[0],uvw[-1])

        plts = [[[0,u[0]],[0,u[1]],[0,u[2]],'b'] for u in uvw ]
        plts +=[[ [0,ez[0]],[0,ez[1]],[0,ez[2]] , 'r' ]]
        dsp.stddisp(plots=plts,labs=['x','y','z'],rc='3d',lw=4)

    def show_xyz(self,**kwargs):
        x,y,z = self.XYZ
        fig2,ax = dsp.stddisp(rc='3d',scat=[x,y,z,2,'b'],labs=['x','y','z'],**kwargs)
        h3d2 = h3D.handler_3d(fig2,persp=False)

    def show_hkl(self,**kwargs):
        h,k,l,I = self.HKL[['h','k','l','I']].values.T
        fig2,ax = dsp.stddisp(rc='3d',scat=[h,k,l,I,'b'],labs=['h','k','l'],**kwargs)
        h3d2 = h3D.handler_3d(fig2,persp=False)

    def mp4_crystal_rotation(self,name,**kwargs):
        cif_file = self.cif_file
        uvw = [self.get_beam_dir(frame=f) for f in np.arange(self.nFrames)+1]
        for i,n in enumerate(uvw[:1]):
            frame = i+1
            name_x = 'figures/%s_x_%s.png' %(name,str(frame).zfill(3))
            name_y,name_z = name_x.replace('_x_','_y_'),name_x.replace('_x_','_z_')
            mut.show_cell(cif_file,name=name_x,title='%d' %frame,n=n,view=[0,0]  ,h3D=0,opt='sc',**kwargs)
            mut.show_cell(cif_file,name=name_y,title='%d' %frame,n=n,view=[90,0] ,h3D=0,opt='sc',**kwargs)
            mut.show_cell(cif_file,name=name_z,title='%d' %frame,n=n,view=[90,90],h3D=0,opt='sc',**kwargs)

        cmd  = '/bin/bash -i -c "im2mp4 figures/%s_x_%%03d.png figures/x.mp4" && ' %name
        cmd += '/bin/bash -i -c "im2mp4 figures/%s_y_%%03d.png figures/y.mp4" && ' %name
        cmd += '/bin/bash -i -c "im2mp4 figures/%s_z_%%03d.png figures/z.mp4" '    %name
        p = Popen(cmd,shell=True,stdout=PIPE,stderr=PIPE);p.wait()
        o,e = p.communicate();
        print(o.decode())

    ########################################################################
    ### def : Viewer
    ########################################################################
    def show_exp(self,frame=1,**kwargs):
        mut.Viewer(self.path+'/tiff',frame=frame,**kwargs)
    def show_sim(self,frame=1,**kwargs):
        mut.Viewer(self.path+'/multislice',frame=frame,**kwargs)
    def show_frames(self,thick=1000,Nmax=5,Smax=0.02,e0=[1,0,0],rot=0,Imag=10,Itol=20,opts='PKqr',**sargs):
        kargs = dict(zip(
            ['Nmax','Smax','e0','rot','Itol','opts'],
            [Nmax,Smax,e0,rot,Itol,opts]))
        return mut.Frames_Viewer(self,thick,Imag,kargs,**sargs)


    def show_frame(self,frame=0,opts='Pqr',
        thick=1000,Nmax=5,Smax=0.02,e0=[1,0,0],rot=0,Imag=10,Itol=20,
        show_hkl=True,qopt=True,rings=True,sim_pattern=None,
        **kwargs):
        ''' Show a frame with information specified by opts
        - opts : E(exp), P(proc), S(sim), K(kin), h(hkl),q(rec A),r(rings), k(hkl_k)
        '''
        if isinstance(opts,str):exp,proc,sim,kin,show_hkl,qopt,rings,show_hkl_kin = [c in opts for c in 'EPSKhqrk']
        if kin:qopt=1

        plts,scat,txts,labs,qmax,wm = [[0,0,'b+']],(),[],['px','py'],0,5
        # plts=[]
        if qopt :
            wm*=self.aper
            labs = ['$q_x$','$q_y$']
        if proc:
            rpl0  = self.rpl.loc[self.rpl.F==frame]
            I,Im = rpl0[['I','Im']].values.T
            if qopt :
                px,py = rpl0[['qx','qy']].values.T
                # px,py = rpl0[['rpx','rpy']].values.T
                # cx,cy = self.cen.iloc[frame-1][['px','py']].values.T
                # px,py = ((np.vstack([px,-py]).T-[cx,-cy])*self.aper).T
            else:
                px,py = rpl0[['rpx','rpy']].values.T
                cx,cy = self.cen.iloc[frame-1][['px','py']].values.T
                plts = [[cx,cy,'b+']]
            c = (0.5,)*3
            scat += ([px,py,I,c,'o'],)
            qmax = np.ceil(max(py.max(),px.max()))

            if show_hkl:
                hkl1  = rpl0[['h','k','l']]
                h,k,l = hkl1.values.T
                hx,kx,lx = rpl0[['hx','kx','lx']].values.T
                sx = lambda x:['%d' %round(x),'%.1f' %x][abs(x-round(x))>0.06]
                txts += [[x+0*wm,y+wm,'(%s,%s,%s)' %(sx(h0),sx(k0),sx(l0)),(0.5,)*3] for x,y,h0,k0,l0 in zip(px,py,hx,kx,lx)]
                print(h,k,l)

        if kin:
            px_k,py_k,I_k,hkl_k = self.get_kin(frame,thick,Nmax,Smax,e0,rot,Imag)
            scat += ([px_k,py_k,I_k,'g','o'],)
            if show_hkl_kin:
                h,k,l = hkl_k
                Itol = I_k.max()/Itol
                txts += [[x,y+wm,'(%s,%s,%s)' %(h0,k0,l0),'g'] for x,y,h0,k0,l0,I in zip(px_k,py_k,h,k,l,I_k) if I>Itol]
                print(h,k,l)
            qmax = np.ceil(max(qmax,px_k.max(),py_k.max()))

        im,bgcol = None,'k'
        if sim:
            qx,qy,I=sim_pattern
            qmax = np.ceil(max(qmax,qx.max(),qy.max()))
            im,bgcol=[qx,-qy,I],None
            # print(bgcol)

        if rings and qopt:
            t = np.linspace(0,np.pi*2,100)
            ct,st = np.cos(t),np.sin(t)
            plts+=[[i*ct,i*st,'m','',0.5] for i in np.arange(0.25,qmax,0.25)]
            plts+=[[i*ct,i*st,'m','',2] for i in np.arange(1,qmax)]
        # if rot:
        #     ct,st = np.cos(np.deg2rad(rot)),np.sin(np.deg2rad(rot))
        #     px,py = ct*px-st*py,st*px+ct*py


        if not 'fonts' in kwargs.keys():kwargs['fonts']={'text':15}
        if not 'xylims' in kwargs.keys():kwargs['xylims']=[-px.max(),px.max(),-py.max(),py.max()]
        dsp.stddisp(plts,ms=20,scat=scat,im=im,texts=txts,bgcol=bgcol,gridOn=0,
            labs=labs,sargs={'alpha':0.5},**kwargs)

    ###########################################################################
    #### compare :
    ###########################################################################
    def rotate_pattern(self,frame,qxqy):
        xyz0  = np.vstack([qx,qy,np.zeros(qx.shape)])

        alpha,beta,omega = -self.cif.iloc[frame-1][['alpha','beta','omega']]

        R = np.identity(3)
        omega_r = np.deg2rad(omega)
        ct,st  = np.cos(omega_r),np.sin(omega_r)
        Rz = np.array([[ct,st,0],[-st,ct,0],[0,0,1]]) #get_crystal_rotation(u=[0,0,-1],alpha=self.omega)
        R = Rz.dot(R)
        alpha_r = np.deg2rad(alpha)
        ct,st   = np.cos(alpha_r),np.sin(alpha_r)
        Rx = np.array([[1,0,0],[0,ct,st],[0,-st,ct]])
        R  = Rx.dot(R)
        beta_r = np.deg2rad(beta)
        ct,st   = np.cos(beta_r),np.sin(beta_r)
        Ry = np.array([[ct,0,st],[0,1,0],[-st,0,ct]])
        R  = Ry.dot(R)

        x0,y0,z0  = R.dot(xyz0)


    def compare_xyz_pxpy(self,frame=32,opts='oa',view=[90,90],**kwargs):
        rpl0    = self.rpl.loc[self.rpl.F==frame]
        pxc,pyc = self.cen.iloc[frame-1][['px','py']].values.T
        omega,beta,alpha = self.omega,0,rpl0.alpha.iloc[0]


        px,py = rpl0[['rpx','rpy']].values.T
        px    =  self.aper*(px-pxc)
        py    = -self.aper*(py-pyc)
        xyz0  = np.vstack([px,py,np.zeros(px.shape)])

        R = np.identity(3)
        if 'o' in opts:
            omega_r = np.deg2rad(omega)
            ct,st  = np.cos(omega_r),np.sin(omega_r)
            Rz = np.array([[ct,st,0],[-st,ct,0],[0,0,1]]) #get_crystal_rotation(u=[0,0,-1],alpha=self.omega)
            R = Rz.dot(R)
        if 'a' in opts:
            alpha_r = np.deg2rad(alpha)
            ct,st   = np.cos(alpha_r),np.sin(alpha_r)
            Rx = np.array([[1,0,0],[0,ct,st],[0,-st,ct]])
            R  = Rx.dot(R)
        if 'b' in opts:
            beta_r = np.deg2rad(beta)
            ct,st   = np.cos(beta_r),np.sin(beta_r)
            Ry = np.array([[ct,0,st],[0,1,0],[-st,0,ct]])
            R  = Ry.dot(R)

        x0,y0,z0  = R.dot(xyz0)

        xyz   = rpl0[['x','y','z']].values.T
        x,y,z = xyz

        Im = np.array(rpl0.Im.values/10,dtype=int)
        if isinstance(view,str):
            if   view=='x':labs,scat = ['y','z'],([y,z,Im,'b','o'],[y0,z0,Im,'r','o'])
            elif view=='y':labs,scat = ['x','z'],([x,z,Im,'b','o'],[x0,z0,Im,'r','o'])
            elif view=='z':labs,scat = ['x','y'],([x,y,Im,'b','o'],[x0,y0,Im,'r','o'])
            dsp.stddisp(xylims=1.5,scat=scat,labs=labs,legElt={'pxpy':'bo','xyz':'ro'},**kwargs)
        else:
            scat = ([x,y,z,Im,'b','o'],[x0,y0,z0,Im,'r','o'])
            fig,ax  = dsp.stddisp(rc='3d',view=view,xylims=1.5,scat=scat,labs=['x','y','z'],**kwargs)
            h3d = h3D.handler_3d(fig,persp=False)

    def compare_hkl(self,frame,eps=None,Smax=0.025,Nmax=5,v=0):
        uvw  = self.get_beam_dir(frame=frame)
        df   = self.get_excitation_errors(frame,Nmax=Nmax,Smax=Smax)
        hkl0 = df[['h','k','l']]

        hkl1 = self.rpl.loc[self.rpl.F==frame] #[['hx','kx','lx']].values
        if eps:
            idx  = (abs(hkl1.hx-hkl1.h)<eps) & (abs(hkl1.kx-hkl1.k)<eps) & (abs(hkl1.lx-hkl1.l)<eps)
            hkl1 = hkl1.loc[idx]
        hkl1 = hkl1[['h','k','l']]

        rrmv = [r for r in hkl1.values if not np.where(np.linalg.norm(r-hkl0.values,axis=1)==0)[0].size]
        rrmv = np.array(rrmv,dtype=int)
        print('%d reflections in diffraction patterns ' %hkl1.shape[0])
        print('%d reflections within Smax=%.2E of the Ewald sphere ' %(hkl0.shape[0],Smax))
        print('%d reflections in diffraction pattern not in ewald construction ' %(rrmv.shape[0]))
        if rrmv.shape[0]:print('\tdetails : ',rrmv)

        ridx = []
        for r in hkl1.values:
            idx = np.where(np.linalg.norm(r-hkl0.values,axis=1)==0)[0]
            if idx.size:
                ridx+=[idx[0]]
        if v:
            print('Reflections from diffraction patterns :');print(hkl1)
            print('Reflections from excitation errors of Ewald construction :');
            print(df.iloc[ridx][['h','k','l','Sw']])
        return df.iloc[ridx][['h','k','l','Sw']]

    def check_orientation_matrix(self):
        lab = np.identity(3)
        cry = self.A.dot(self.lattice_vectors)
        # cr2 = self.A.dot(self.reciprocal_vectors).T).T
        plts  = [ [[0,ai[0]],[0,ai[1]],[0,ai[2]],c,'$%s$' %l]  for i,(ai,c,l) in enumerate(zip(lab,['r','g','b'],['x','y','z'])) ]
        # plts += [ [[0,ai[0]],[0,ai[1]],[0,ai[2]],[c,'--'],'$%s$' %l]  for i,(ai,c,l) in enumerate(zip(cry,['r','g','b'],['a','b','c'])) ]
        x0,y0,z0 = [0.5]*3
        plts += [ [[x0,ai[0]+x0],[y0,ai[1]+y0],[z0,ai[2]+z0],[c,'--'],'$%s^{*}$' %l]  for i,(ai,c,l) in enumerate(zip(cr2,['r','g','b'],['a','b','c'])) ]
        # dsp.stddisp(plts,rc='3d',view=[0,0],name='figures/glycine_orient.png',opt='sc')



import importlib as imp
from utils import*
import multislice.pymultislice as ms;imp.reload(ms)
import multislice.multi_3D as MS3D  ;imp.reload(MS3D)
import multislice.mupy_utils as mut ;imp.reload(mut)
from crystals import Crystal
plt.close('all')
path='../../multislice/docs_fig/multi3D/'
new = 1
name = 'dat/test.pkl'


#cif file for alpha-glycine from :
#https://pubs.acs.org/doi/suppl/10.1021/cg101059c/suppl_file/cg101059c_si_002.pdf
cif_file = 'dat/alpha_glycine.cif'
# xyz_file = cif_file.replace('.cif','.xyz')
crys = Crystal.from_cif(cif_file)
lat_vec = np.array(crys.lattice_vectors)
pattern = mut.import_cif(cif_file)#,xyz=xyz_file)#

pattern[:,3]-=pattern[:,3].min()

lat_params = lat_vec.max(axis=0)-lat_vec.min(axis=0)
# mut.plot_grid(pattern,lat_params,'xy',equal=True,ms=20)
# mut.plot_grid(pattern,lat_params,'xz',equal=True,ms=20)
# mut.plot_grid(pattern,lat_params,'yz',equal=True,ms=20)


if new:
    ax,by,cz = lat_params
    mp = MS3D.Multi3D(pattern,ax,by,cz,
        keV=200,Nxy=[3,2],nxy=2**10,Nz=1,dz=1,
        TDS=False,
        iZs=1,iZv=1,opts='q',v=1,
        hk=3,hkopt='sro',
        eps=1,copt=0,
        ppopt='B')#XQZTP)
    mp.save(name)

else:
    mp = ms.load(name)

mp.Bz_show(cm='jet')
# mp.Vz_show(iz=0,cmap='viridis')
# mp.Vz_show(iz=1,cmap='viridis')
# mp.Vz_show(iz=2,cmap='viridis')



from utils import*

def plot_Ikin_Idyn(opt='p'):
    txi = [0.1,0.5]
    t   = [100,500]                   #A
    xi  = np.array(t)/np.array(txi) #A
    K   = 100                       #A^-1
    Ug  = K/np.array(xi)            #A^-2
    print('xi(A):',xi);print('Ug(A^-2):',Ug)
    w = np.linspace(-10,10,1000)
    Ikin = lambda t,xi:np.sinc(t/xi*w)**2
    Idyn = lambda t,xi:np.sinc(t/xi*np.sqrt(1+w**2))**2
    plts = [[w,Ikin(t[0],xi[0]),'b--',''],[w,Idyn(t[0],xi[0]),'b',''],
            [w,Ikin(t[1],xi[1]),'r--',''],[w,Idyn(t[1],xi[1]),'r','']]
    legElt = {'kin':'k--','dyn':'k',
             't=%dA' %int(t[0]):'b', 't=%dA' %int(t[1]):'r'}
    dsp.stddisp(plts,labs=[r'$w (A^{-1})$' ,'$I(w)/\sigma v_g t$'],
        lw=2,opt=opt,figsize='12',legElt=legElt,name=figpath+'kin_dyn.svg')

def plot_Ikin_Idyn_S0(opt='p'):
    K  = 100        #A^-1
    Ug = 0.1        #A^-2
    tlam = K/Ug
    t    = [10,100,500,1000,2000]
    txi  = np.array(t)/tlam; #
    print('xi   = %.2f A' %(tlam))
    print('t(A) =',t)
    print('txi  =',txi)
    if 'p' in opt:
        t = np.linspace(0,1000,1000) #A
        t0 = np.array([100,500])
        Ikin = lambda t:(pi*t/tlam)**2
        Idyn = lambda t:np.sin(pi*t/tlam)**2
        plts = [[t,Ikin(t),'k--','kinematic'],[t,Idyn(t),'k','dynamic']]
        plts+= [[t0[0],Ikin(t0[0]),'bs',''],[t0[0],Idyn(t0[0]),'bs','']]
        plts+= [[t0[1],Ikin(t0[1]),'rs',''],[t0[1],Idyn(t0[1]),'rs','']]
        dsp.stddisp(plts,labs=['$Thickness$ (A)' ,'$I(S_G=0)$'],
            lw=2,opt=opt,figsize='12',name=figpath+'kin_dyn0.svg')

####### display
def plot_Idyn():
    tzeta = [0.1,0.25,0.5,0.75,1,1.5]#,2,2.5,3];
    tzeta = [0.75,1.5]
    nTs=len(tzeta)
    cs = getCs('Reds',nTs)
    w = np.linspace(-6,6,1000)
    I = lambda tz:np.sin(pi*tz*np.sqrt(1+w**2))**2/(1+w**2)
    plts=[[w,I(tzeta[i]),cs[i],'%.2f' %(tzeta[i])] for i in range(nTs)]
    dsp.stddisp(plts,labs=[r'$\omega$','$Intensity$'],title=r'$t/\xi$',lw=2)#,xylims=[-6,6,0,1])

#plot_Idyn()
plot_Ikin_Idyn_S0(opt='s')
plot_Ikin_Idyn(opt='s')



import importlib as imp
from utils import*
import multislice.multi_2D as MS2D  ;imp.reload(MS2D)
import wallpp.plane_group as pg     ;imp.reload(pg)
plt.close('all')
path='../multislice/docs_fig/multi2D/'

keV = 100#200
ndeg = 2**8
eps = 0.01
#### Potential of a structure from wallpaper library
pptype,a,b,angle = 'p1',10,4,90
# pattern = np.array([[2,2,4]])
pattern = np.array([[2,2,1]])
p1      = pg.Wallpaper(pptype,a,b,angle,pattern,ndeg=ndeg)
pattern = p1.get_potential_grid_p1()


Nx=1
iZv = np.inf
def base():
    mp0 = MS2D.Multi2D(pattern,a,b,keV,
            Nx=Nx,dz=b,nz=100,ppopt='',#XQZTP
            TDS=False,
            iZs=1,iZv=10,opts='q',eps=eps)

    # mp0.Bz_show(iBs=np.arange(6)*Nx,lw=2,xylims=[0,400,0,1])
    # mp1.propagate(300,iZs=2)
    # mp0.Xxz_show(pOpt='pt',axPos=[0.14,0.12 ,0.85,0.85],
    #     opt='p',name='../docs_fig/multislice/multi_2D.png')
    # mp1.propagate(10,iZs=2)
    # mp1.save('test.pkl')
    # mp2 = MS2D.load('test.pkl')
    # mp2.Bz_show()
    mp0.Qz_show([0,-1],opts='S')
    mp0.Qxz_show(iZs=5)
    # mp0.Qz_show(slice(0,10,2))
    return mp0

def ms_sg():
    '''Difference between choosing propagator with + and with -'''
    mp0 = MS2D.Multi2D(pattern,a,b,keV,
            Nx=Nx,dz=b,nz=0,ppopt='',#XQZTP
            iZs=1,iZv=10,eps=eps)
    mp1 = MS2D.Multi2D(pattern,a,b,keV,
            Nx=Nx,dz=b,nz=0,ppopt='',#XQZTP
            iZs=1,iZv=10,eps=eps)
    mp0._set_propagator(sg=1)
    mp1._set_propagator(sg=-1)
    mp0.propagate(1000)
    mp1.propagate(1000)
    # plts=[[mp0.q,mp1.getI()-mp0.getI(),'b','']]#,[mp1.q,mp1.getI(),'r','-']]
    plts=[[mp0.z,mp0.getB(1),'b','+'],[mp1.z,mp1.getB(1),'r','-']]
    dsp.stddisp(plts)

def small_thick():
    mp1 = MS2D.Multi2D(pattern,a,b,keV,
            Nx=Nx,dz=0.1*b,nz=1000,ppopt='',#XQZTP
            iZs=1,iZv=iZv,eps=eps)
    # mp1.Bz_show(iBs=np.arange(6)*Nx,lw=2,xylims=[0,400,0,1])
    return mp1

def tilt(t=0.1,eps=eps):
    mpt = MS2D.Multi2D(pattern,a,b,keV,tilt=t,
            Nx=Nx,dz=0.1*b,nz=1000,ppopt='',#XQZTP
            iZs=1,iZv=1,eps=eps,v=0)
    # mp1.Bz_show(iBs=np.arange(6)*Nx,lw=2,xylims=[0,400,0,1])
    return mpt

def tilts_test(tilts=np.linspace(0,0.9,100),
    eps=0.01,nz=3000):
    mp2 = np.empty(tilts.size,dtype=object)
    for i,t in enumerate(tilts):
        print('theta=%.2f' %t)
        mp2[i] = MS2D.Multi2D(pattern,a,b,keV,tilt=-t,
                Nx=1,dz=0.1*b,nz=nz,ppopt='',#XQZTP
                iZs=10,iZv=100,eps=eps,sg=-1,v=1)
    return tilts,mp2

mp0=base()
# ms_sg()
# mp1=small_thick()
# mpt=tilt(t=0.1)
# ts,msts=tilts_test(tilts=np.linspace(0,0.2,51),eps=0.1,nz=10010)
# ts,msts=tilts_test(tilts=np.linspace(0,0.3,21)[6:8],eps=0.02,nz=50000)
# tilts_show(ts,msts,iBs=1,iZs=-1,name=path+'rocking_eps1.svg',opt='p')
# tilts_show(msts,iBs=np.arange(-3,3),iZs=-1,name=path+'rocking_eps1.svg',opt='p')
# MS2D.tilts_show(ts,msts,iBs=1,iZs=slice(0,10000,1000),name=path+'rocking_eps1.svg',opt='p')
# m0 = msts[25];m0.Bz_show(np.arange(-5,5))
# msts[10].Ewald_show(lw=2,xylims=[-0.01,0.4,-0.001,0.005])
# msts=tilts_test(tilts=np.linspace(0,0.2,51),eps=0.10,name=path+'rocking_eps2.svg',opt='ps')



from utils import*
plt.close('all')

opts = '1'

if '1' in opts:
    dfs = np.arange(-100,101,50)
    Cs = np.array([1,2])*1e6 #nm
    keV = np.array([100,200])
    lam = cst.keV2lam(keV)*0.1 #nm

    u = np.linspace(0,6,1000) #nm^-1

    Xi = lambda lam,df,Cs:(df*lam*u**2 + 0.5*Cs*lam**3*u**4)*np.pi
    if 'S' in opts:Xi = lambda lam,df,Cs: np.sin((df*lam*u**2 + 0.5*Cs*lam**3*u**4)*np.pi)

    cs00 = dsp.getCs('Reds',dfs.size)
    cs10 = dsp.getCs('Blues',dfs.size)
    cs01 = dsp.getCs('Greens',dfs.size)

    plts00 =[[u,Xi(lam[0],df,Cs[0]),cs00[i],r'$\Delta f=%d$' %df] for i,df in enumerate(dfs)]
    plts10 =[[u,Xi(lam[1],df,Cs[0]),cs10[i],r'$\Delta f=%d$' %df] for i,df in enumerate(dfs)]
    plts01 =[[u,Xi(lam[1],df,Cs[1]),cs01[i],r'$\Delta f=%d$' %df] for i,df in enumerate(dfs)]
    laby = '$\chi(u)$'
    if 'S' in opts : laby = '$\sin\chi(u)$'
    dsp.stddisp(plts00,lw=2,labs=['$u(nm^{-1})$',laby],title='$\lambda=%dkeV, Cs=%dmm$' %(keV[0],Cs[0]*1e-6))
    dsp.stddisp(plts10,lw=2,labs=['$u(nm^{-1})$',laby],title='$\lambda=%dkeV, Cs=%dmm$' %(keV[1],Cs[0]*1e-6))
    dsp.stddisp(plts01,lw=2,labs=['$u(nm^{-1})$',laby],title='$\lambda=%dkeV, Cs=%dmm$' %(keV[1],Cs[1]*1e-6))

if '2' in opts:
    lam = cst.keV2lam(200)
    beta = np.linspace(5,50,100) #mrad
    beta0 = np.array([5,10,20,40])
    plts = [[beta,1.22*lam/beta*1e3 ,'b']]
    plts +=[[beta0,1.22*lam/beta0*1e3,'b*']]
    # plts += [beta,1.22*lam/beta*1e3,'b']
    dsp.stddisp(plts,labs=[r'$\beta(mrad)$',r'$d(\AA)$'],lw=2,ms=10,xylims=[1,51,0.5,6.5])



from utils import*
from blochwave import bloch_old as bw;imp.reload(bw)

def test_2beams3D(k0=100,N=2):
    # k0 = 100 #A^-1 => lambda=0.01
    hkl,Gs,Vg = get_pattern3D(N)
    # idxz = np.abs(Gs[:,1])<1e-3; #print(hkl[idxz][:,[0,2]])
    # Gs = Gs[idxz][:,[0,2]]

rot  = lambda t:np.array([[np.cos(t),np.sin(t)],[-np.sin(t),np.cos(t)]])
def test_2beams(N,s,eta=1,Ts=np.linspace(1,100,100),opts=''):
    ''' Test a 2-beam configuration
    - N : number of beams such that Vg.shape= 2N+1 x 2N+1
    - s : slices of Vg beams for the G beams to perform blochwave calculation
    - np.s_[...] - actual slice
    - int - so that s=np.s_[2*N,2*N-2:2*N+s]
    - Ts : int or np.ndarray thickness
    - eta : misaligment
    - opts : T(thickness), E(Ewald), F(structure factor)
    returns :
    - St : Nbeams x Ts.size array - intentities of beams at the different thicknesses
    '''
    k0 = 5.008343065817388
    if isinstance(s,int) :
        iS = s
        if iS<=2 : s = np.s_[2*N, [2*N-2,2*N]]
        if iS>2  : s = np.s_[2*N, 2*N-2:2*N-2+iS]

        #### basic pattern
        hl,Gs,Fhl = bw.get_pattern2D(N,s)
        gx0,gz0 = Gs.T
        #orient in a 2-beam excitation setup
        gx1,gz1 = rot(eta/10).dot(np.stack([gx0.flatten(),gz0.flatten()]))
        Gs = np.array([gx1,gz1]).T

        #plot reciprocal space lattice
        Sg = (k0**2-np.linalg.norm(k0*np.array([0,1])-Gs,axis=1)**2)/(2*k0)
        if 'E' in opts:
            t = np.linspace(0,2*np.pi,500)
            plts = [[k0*np.cos(t),k0*np.sin(t)+k0,'r','']]
            scat = [np.hstack([gx0,gx1]),np.hstack([gz0,gz1]),['k']*gx0.size+['b']*gx1.size]
            txts = [[g[0],g[1],'%.2f'%s,'b'] for g,s in zip(Gs,Sg)]
            a = 1.5
            dsp.stddisp(plts,texts=txts,scat=scat,ms=20,xylims=[-a,a,-a,a],lw=2,fonts={'text':10})

        #### structure factor
        Fhl[Fhl==Fhl.max()]=0
        if 'F' in opts:
            dsp.stddisp(im=[abs(Fhl)],imOpt='cv',axPos='V',pOpt='t')

        #### SOLVE
        gammaj,CjG = bw.solve_Bloch(k0,hl,Gs,Fhl,v=0)

        #### scattering matrix
        if isinstance(Ts,float) or isinstance(Ts,int) : Ts=np.array([Ts])
        if isinstance(Ts,list):Ts=np.array(Ts)
        Ng = Gs.shape[0]
        St = np.zeros((Ng,Ts.size),dtype=complex)
        for iT,T in enumerate(Ts):
            # S=CjG.T.dot(np.diag(np.exp(2J*np.pi*gammaj*T))).dot(CjG)
            S=CjG.dot(np.diag(np.exp(2J*np.pi*gammaj*T))).dot(CjG.T)
            # St[:,iT]=CjG.dot(np.diag(np.exp(2J*np.pi*gammaj*T))).dot(CjG.T)[0,:]
            St[:,iT] = S[:,0]
        #display thickness dependent intensities
        if 'T' in opts:
            It = np.abs(St)**2
            cs = dsp.getCs('jet',Ng)
            plts = [[Ts,It[iG],[cs[iG],'-o'],'%s' %hl_iG] for iG,hl_iG in enumerate(hl)]
            dsp.stddisp(plts,labs=[r'Thickness $(\AA)$','$Ig$'],gridmOn=1)
        return St,Sg


def test_2beams_rocking(etas=np.linspace(0.5,1.5,10),Ts=100,opts='wz'):
    if isinstance(Ts,float) or isinstance(Ts,int) :Ts=np.array([Ts])
    if isinstance(Ts,list):Ts=np.array(Ts)

    Fg = np.zeros((Ts.size,etas.size),dtype=complex)
    Sg = np.zeros((etas.shape))
    for iE,eta in enumerate(etas):
        St,Sg0 = test_2beams(N=2,s=2,eta=eta,Ts=Ts,opts='')
        Fg[:,iE] = St[1,:]
        Sg[iE] = Sg0[0]
        It = np.abs(Fg)**2
    if 'w' in opts:
        cs = dsp.getCs('Spectral',Ts.size)
        plts = [[Sg,It[iT],[cs[iT],'-o'],'$%.1fA$' %T] for iT,T in enumerate(Ts)]
        dsp.stddisp(plts,labs=['$s_g$','$I_g$'],lw=2)
    if 'z' in opts:
        xi_g=525
        cs = dsp.getCs('Spectral',Sg.size)
        plts = [[Ts,It[:,iS],[cs[iS],'-o'],'$w_g=%.1f$' %(Sg0*xi_g)] for iS,Sg0 in enumerate(Sg)]
        dsp.stddisp(plts,labs=['$s_g$','$I_g$'],lw=2)

# plt.close('all')
# St = test_2beams(N=2,s=2,eta=1,Ts=np.linspace(0,1000,100),opts='T')
# St = test_2beams(N=2,s=3,eta=1,Ts=np.linspace(0,1000,100),opts='ET')
# St = test_2beams(N=2,s=5,eta=1,Ts=np.linspace(0,1000,100),opts='T')
# St = test_2beams(N=2,s=5,eta=1,Ts=np.linspace(0,1000,100),opts='T')
# St = test_2beams(N=2,s=5,eta=1.5,opts='ET')

# test_2beams_rocking(etas=np.linspace(1.0,1.05,4),Ts=np.linspace(0,1000,500),opts='z')
test_2beams_rocking(etas=np.linspace(0.95,1.05,200),Ts=np.array([0.5,0.75,1,1.25,1.5])*525,opts='w')



import importlib as imp
from utils import*                        ;imp.reload(dsp)
import multislice.multislice as mupy      #;imp.reload(mupy)
import multislice.postprocess as pp       #;imp.reload(pp)
import multislice.mupy_utils as mut       ;imp.reload(mut)
import multislice.rotating_crystal as rcc #;imp.reload(rcc)
plt.close('all')

def test_base(name,**kwargs):
    multi=mupy.Multislice(name,keV=200,
        repeat=[1,1,1],NxNy=512,slice_thick=1.5,Nhk=6,
        **kwargs)
    return multi


def show_postprocess(tag):
    multi=pp.load(name,tag)
    multi.beam_vs_thickness(tol=1e-4)
    multi.pattern(Iopt='Isnlg',tol=1e-4,rings=[0,0.25,0.5,1],caxis=[-6.2,0],
        pOpt='pt',cmap='binary',imOpt='ch',axPos=[0.2,0.13,0.75,0.75])

def test_get_tilts():
    tests = [
    'mupy.get_tilts(tx=np.arange(0,10,0.1))',
    'mupy.get_tilts(tx=np.arange(0,10,0.1),ty=0.1)',
    'mupy.get_tilts(tx=0.1,ty=np.arange(0,10,0.1))',
    'mupy.get_tilts(tx=np.arange(0,10)*0.2,ty=np.arange(0,10)*0.1)',
    'mupy.get_tilts(tx=np.arange(0,10,0.2),ty=np.arange(0,10,0.1))',
    'mupy.get_tilts(tx=0.1,ty=0)',]
    for test in tests:
        try:
            print(colors.green+test+colors.black)
            eval(test)
        except Exception as e:
            print(colors.red, e,colors.black)


if __name__ == '__main__':
    name  = '../dat/test/'

    # file = '../../tests/multislice/dat/alpha_glycine.cif'
    # xyz = 'dat/alpha_gly.xyz'
    file = 'diamond'
    xyz = name+'diamond_2beams.xyz'
    # crys = mut.import_crys(file)
    lat_params = [10,10,20]
    u = [0.13846063, 0.01406432, 0.99026807]
    mut.gen_xyz2(file,xyz,lat_params,n=u,theta=0,pad=1,opts='p')
    # mut.gen_xyz('Si',n=[1,1,1],theta=0,rep=[20,20,100],pad=[2,2,0],xyz=xyz)
    # mut.show_grid(xyz,opts=['xy','xz'],popts='hvp',figs='f')#,xylims=[0,1000,0,1000])
    # mut.show_grid(xyz,opts='yz')
    # mut.show_grid(xyz,opts='xz')#,xylims=[])
    # mut.show_grid3(xyz)
    # mut.show_cell(file,n=[1,1,1])

    #
    multi = test_base(name,mulslice=False,opt='dsrfp',ppopt='ws',tag='base',i_slice=20,v=1)
    # multi = test_base(name,mulslice=False,opt='dsrp',fopt='f',ppopt='uwB',v=2,ssh='tarik-CCP4home')
    # multi = test_base(name,mulslice=False,opt='dsrp ',fopt='f',ppopt='uwBP',v=2,ssh='badb')
    # multi = test_base(name,mulslice=False,fopt='f',opt='dsr',ppopt='',ssh='tarik-CCP4home',v='nctrdDR')
    # multi = test_base(name,mulslice=True,ssh=None,fopt='f',opt='dsr',v='nctrdDR')
    # multi = test_base(name,tag='TDS',mulslice=False,TDS=True,T=300,fopt='f',ppopt='wP',opt='dsrp',v='nctrdDR')
    # show_postprocess(tag,'')

    # test_get_tilts()
    # rock = mupy.Rocking(name,tx=np.arange(3)*0.05,ty=0,tag='tx',
    #     NxNy=256,repeat=[1,1,10],Nhk=3,
    #     opt = 'sr')
    # rock = pp.rock_load(name,'tx')
    # rock.update(v=1);
    # rock.plot_rocking(iBs=[(1,1),(0,1)],iZs=None,zs=[5,15,38])


    # multi = pp.load(name,tag='base',v=2)
    # multi.resume(opt='srfp',ppopt='w',Nz=30,i_slice=20)#Nhk=4)
    # multi.save_pattern(iz=15)
    # multi.pattern(Iopt='ncsgt',iz=5,Nmax=64,Imax=3e4,gs=0.3,
    #     rmax=100,tol=1e-3,v=1,)#xylims=2,caxis=[0,0.1])
    # multi.pattern(Iopt='ncsgt',rot=2,iz=5,Nmax=64,Imax=3e4,gs=0.3,rmax=100,tol=1e-3,v=1,)
    # qx,qy,I = multi.pattern(Iopt='cns',gs=2,rmax=20,out=1)
    # ss = [np.s_[i+int(qx.shape[0]/2),:] for i in range(4)]
    # cs = dsp.getCs('Blues',4)
    # dsp.stddisp([[qx[s],I[s],cs[i],'%d' %i] for i,s in enumerate(ss)] )
    # multi.get_beams(bOpt='fa');
    # multi.merged=0
    # multi.merge_beams()
    # multi.beam_vs_thickness(orig=1,tol=1e-3)
    # multi.pattern(iz=15,Iopt='sNc',Nmax=100,cmap='viridis',caxis=[0,0.25],xylims=5)
    # multi.patterns2gif(Iopt='scN',Nmax=100,xylims=2,caxis=[0,0.25])#,pOpt='X')
    # multi.show_patterns(Iopt='s',caxis=[0,1e10],xylims=5)



import importlib as imp
# from subprocess import Popen,PIPE
from utils import*                      ;imp.reload(dsp)
import multislice.mupy_utils as mut     ;imp.reload(mut)
import multislice.pets as pp            ;imp.reload(pp)
plt.close('all')
pts_path = 'glycine/glycine.pts'
cif_file = 'glycine.cif'


pets = pp.Pets(pts_path,gen=0)#,lam=0.02508,aper=0.005340,omega=230,gen=1)
frame = 65

# pets.show_exp(frame,v=0)
# pets.show_sim()

# pets.compare_xyz_pxpy(frame=frame,opts='oab')

# pets.show_uvw()
# pets.show_xyz()
# pets.show_xyz(view=[0,0],opt='sc',name='pets_glycine_xyz.png')
# pets.show_hkl(view=[0,0],opt='sc',name='pets_glycine_hkl.png')
# pets.show_frame(frame=frame,name='pattern_%d.png' %frame, opt='psc')
# pets.show_ewald_sphere(frame=19,Nmax=5,Smax=0.025,h3d=1)
# pets.show_hkl()
# K = pets.get_beam(frame)
# df = mut.get_kinematic_pattern(cif_file,K,Nmax=5,Smax=0.02)
pets.show_frames(frame=frame,thick=1000,Nmax=7,Smax=0.015,rot=206,Imag=10,opts='PKh',
    v=0,pargs={'xylims':1.5})

# df = pets.compare_hkl(frame,Nmax=8,Smax=0.025,eps=None,v=0)

# frames = range(1,65) #[5,10,15,25,30]
# for f in frames:
#     pets.show_frame(frame=f,Smax=0.01,name='glycine_exp%d.png' %f,opt='sc')

# xylims = [-5,10,-5,10,-12,1]
# mut.show_cell(cif_file,title='%d' %frame,xylims=xylims,n=uvw,
#     axPos=[0,0,1,0.95],view=[0,0],h3D=1,opt='p')
# pets.mp4_crystal_rotation('glycine',xylims=xylims)



from utils import*
plt.close('all')

opts='d'


dfs = np.arange(-100,101,50)
Cs = np.array([1,2])*1e6 #nm
keV = np.array([100,200])
lam = cst.keV2lam(keV)*0.1 #nm

u = np.linspace(0,6,1000) #nm^-1
Xi = lambda lam,df,Cs: np.sin((df*lam*u**2 + 0.5*Cs*lam**3*u**4)*np.pi)

if 'a' in opts:
    cs00 = dsp.getCs('Reds',dfs.size)
    fig,axs = dsp.create_fig(rc=[dfs.size,1],pad=0.0)
    for i,df in enumerate(dfs):
        plts00 =[[u,Xi(lam[1],df,Cs[0]),'b',r'$\Delta f=%dnm$' %df]]
        dsp.stddisp(plts00,fig=fig,ax=axs[i],
            lw=2,opt='',pOpt='ltG',fonts={'tick':10,'leg':10},legLoc='upper left')#title='$\lambda=%dkeV, Cs=%dmm$' %(keV[1],Cs[0]*1e-6))
    axs[0].set_title('$\lambda=%dkeV, Cs=%dmm$' %(keV[1],Cs[0]*1e-6),fontsize=15)
    axs[-1].set_xlabel('$u(nm^{-1})$',fontsize=15)
    fig.show()
if 'b' in opts:
    cs10 = dsp.getCs('Blues',dfs.size)
    cs01 = dsp.getCs('Greens',dfs.size)

    plts10 =[[u,Xi(lam[1],df,Cs[0]),cs10[i],r'$\Delta f=%d$' %df] for i,df in enumerate(dfs)]
    plts01 =[[u,Xi(lam[1],df,Cs[1]),cs01[i],r'$\Delta f=%d$' %df] for i,df in enumerate(dfs)]
    dsp.stddisp(plts10,lw=2,labs=['$u(nm^{-1})$',laby],title='$\lambda=%dkeV, Cs=%dmm$' %(keV[1],Cs[0]*1e-6))
    dsp.stddisp(plts01,lw=2,labs=['$u(nm^{-1})$',laby],title='$\lambda=%dkeV, Cs=%dmm$' %(keV[1],Cs[1]*1e-6))

if 'c' in opts:
    dfs = np.linspace(-100,-50,10)
    for i,df in enumerate(dfs):
        plts00 =[[u,Xi(lam[1],df,Cs[0]),'b',r'$\Delta f=%d$' %df]]
        dsp.stddisp(plts00,lw=2,labs=['$u(nm^{-1})$','$\sin\chi(u)$'],title='$\lambda=%dkeV, Cs=%dmm$' %(keV[1],Cs[0]*1e-6))

if 'd' in opts:
    df = -65
    betas = np.array([5,10])
    us = (betas*1e-3)/lam[1]
    plts00 =[[u,Xi(lam[1],df,Cs[0]),'b',r'$\Delta f=%dnm$' %df]]
    plts00+=[[[u,u],[-1,1],'r',''] for u in us]
    dsp.stddisp(plts00,lw=2,labs=['$u(nm^{-1})$','$\sin\chi(u)$'],title='$\lambda=%dkeV, Cs=%dmm$' %(keV[1],Cs[0]*1e-6))



from utils import*
import importlib as imp
import wallpp.plane_group as pg   ;imp.reload(pg)
import multislice.multi_2D as ms  ;imp.reload(ms)
import multislice.nearBragg as nb ;imp.reload(nb)
plt.close('all')
# path='../multislice/docs_fig/multi2D/MSvsNBstrong_'
path='../multislice/docs_fig/multi2D/'
opts='I'

# Problem
keV   = 200        # wavelength 200keV
Nx    = 10         # Transverse unit cells
Nz    = 100        #
ndeg  = 2**7       # number of pixels
pptype,ax,bz,angle,Za = 'p1',10,5,90,  2
pattern = np.array([[5,2.5,Za]])
eps = 0.1
opt = 'p'


if 'I' in opts :
    Nz = 1000
    sig_e = 0.05 #A
    Nslices = 1
    N = Nslices*Nz
    z,Ncoh = np.zeros((N,1)),np.ones((N,1))
    for i in range(1,Nz):
        za = np.array([2.5])
        Na = np.array([1])      #nb atoms
        for iS in range(Nslices):
            j = i*Nslices+iS    #; print(j)
            z[j] = bz*i+za[iS]
            Ncoh[j]=Ncoh[j-1]*(1-sig_e*Na[iS]/ax)

    le = ax*bz/sig_e; print(le) #Angstrom
    Npoi = np.exp(-z/le)
    z/=10
    plts  = [[z,Ncoh,'b','$P_{coh}$']]
    plts += [[z,Npoi,'b--','$Poisson$']]
    dsp.stddisp(plts,labs=[r'$z(nm)$','$P_{coh}$'],lw=2,
        name=path+'Pcohz.svg',opt='p')

if 'C' in opts:
    p1 = pg.Wallpaper(pptype,ax,bz,angle,pattern,ndeg=ndeg)
    potential = p1.get_potential_grid()
    ms0 = ms.Multi2D(potential,ax,bz,keV=keV,
        Nx=1,nz=Nz,dz=bz,eps=eps,
        iZs=1,iZv=1)
    # ms0.Xxz_show(iZs=slice(0,-1,10),iXs=slice(0,-1,4)   ,name=path+'multi2d_Z%d_pot%d_Q.png' %(Za,i),opt='s',pOpt='pt');plt.close('all')
    # ms0.Tz_show(Vopt='V',lw=2,
    #     name=path+'V.svg',opt=opt)
    # ms0.Qz_show(slice(0,11,2),'S',lw=2,xylims=['x',-2.5,2.5],
    #     name=path+'Iz.svg',opt=opt,axPos='T',setPos=1)
    # ms0.Bz_show(np.arange(1,6),lw=2,
    #     name=path+'Bz.svg',opt=opt)

    #### Near Bragg Nz=10
    qmax,npx  = 1.0,2500
    idx = (ms0.q<=qmax) &(ms0.q>=0)
    q0s = ms0.q[idx]
    # nbG = nb.NearBragg(pattern.T,ax,bz,keV=keV,Nx=Nx,Nz=Nz,eps=eps,
    #     method='Fresnel',q0s=q0s)
    #     # npx=npx,qmax=qmax,)
    nbG = nb.NearBragg(pattern.T,ax,bz,keV=keV,Nx=Nx,Nz=Nz,eps=eps,
        method='Dual',q0s=q0s,)

    # Ims10 = ms0.psi_qz[9,:].copy()/ms0.nx**2*ms0.dq**2;#Im0=Ims[0];
    # Ims10[0]=0
    # Ims10/=Ims10.max();
    # Inb10 = nbG.I.copy()
    # idx   = Inb10.argmax();Inb10[idx]=0;#remove forward propagation
    # Inb10/=Inb10.max() #normalize
    #
    # # plts+=[[nbFr.getQ(),Ifr,'b-','$NB_{kin}$']]
    # plts10 =[[nbG.getQ(),Inb10,'g-+','$NB_{kin}$']]
    # plts10+=[[ms0.q     ,Ims10,'ro','MS']]
    # dsp.stddisp(plts10,labs=['q','I'],lw=2,xylims=['x',0,2.01],
    #     # inset={'axpos':[0.55,0.3,0.4,0.4],'xylims':[1,2,-0.01,0.01],'ms':5,'lw':0.5},
    #     opt='p',name=path+'nz10.svg')
    #
    # #### Near Bragg Nz = 100
    # nbG.replicate(ax,bz,Nx,Nz=100)
    # nbG._Fresnel()

    Ims100 = ms0.psi_qz[-1,:].copy()/ms0.nx**2*ms0.dq**2;
    Ims100[0]=0
    Ims100/=Ims100.max();

    Inb100 = nbG.I.copy()
    idx = Inb100.argmax();Inb100[idx]=0;#remove forward propagation
    Inb100/=Inb100.max() #normalize

    plts100 =[[nbG.getQ(),Inb100,'g-+','$NB_{kin}$']]
    plts100+=[[ms0.q     ,Ims100,'ro','MS']]
    dsp.stddisp(plts100,labs=['$q(\AA^{-1})$','I'],lw=2,xylims=['x',0,1.01],
        # inset={'axpos':[0.55,0.3,0.4,0.4],'xylims':[1,2,-0.01,0.01],'ms':5,'lw':0.5},
        opt=opt,name=path+'I.svg')



from crystals import Crystal,Atom,symmetry_expansion,write_xyz
# import rotating_crystal as rcc; imp.reload(rcc)

cif='225.cif'  #201(SC), 212,225,229

lattice_vectors = np.diag([2,2,2])
cCIF = Crystal.from_cif('../dat/Carbone_SC/'+cif)
# cCIF = Crystal.from_database('Fe')
syms = cCIF.symmetry_operations()
atoms = [Atom(z, c) for z,c in zip(['C','C','C'], [[0.5,0.5,0],[0,0.5,0.5],[0.5,0.,0.5]]) ]
# atoms = [Atom('C', [0.5,0.5,1])]
# atoms = [Atom('C', [1.,1.,1.])]
syms4 = [ np.hstack([np.vstack([s[0],[0,0,0]]), np.vstack([s[1][:,None],1]) ]) for s in syms]
unitcell = symmetry_expansion(atoms,syms4)
C = Crystal(unitcell, lattice_vectors)
print('international_number:', cCIF.symmetry()['international_number'])
print('international_number:', C.symmetry()['international_number'])
print(C)
# write_xyz(C,'test.xyz')



import importlib as imp
import pickle
from scattering.structure_factor import structure_factor3D
import multislice.multislice as mupy; imp.reload(mupy)
import multislice.postprocess as pp ; imp.reload(pp)
import multislice.mupy_utils as mut ; imp.reload(mut)
from crystals import Crystal
from utils import*
plt.close('all')
path = '../dat/epicza/'
figpath = '../docs_fig/'
file = path+'epicza.cif'
opts = 'E '  #E(Ewald) T(tilts) B(Beams)F(structFact)


if 'E' in opts:
    #Ewald configuration [010]
    mut.ewald_sphere(lat_params,lam=cst.eV2mum(12000)*1e4,tmax=60,T=20.,nx=5 ,ny=20,opt='p',name=figpath+'E_MX.png',xylims=[-1,1.01,-0.1,2.01],xyTicks=1,legLoc='upper right',fonts=fs)
    mut.ewald_sphere(lat_params,lam=cst.keV2lam(200)     ,tmax=7 ,T=0.2,nx=20,ny=10,opt='p',name=figpath+'E_ED.png',xylims=[-3,3.01,-0.1,1.01],xyTicks=1,legLoc='upper right',fonts=fs)



from utils import*                  #;imp.reload(dsp)
from EDutils import viewers as vw   ;imp.reload(vw)
plt.close('all')


# tests = range(10,12)#[]
tests = [0]

if -1 in tests:v = vw.Viewer()  #nothing should raise exception
if 0 in tests:v = vw.Viewer(path='dat/glycine',init_opts='',pets_opts='SKr')           #path alone (contains cif_file)
if 1 in tests:v = vw.Viewer(path='dat/pets',init_opts='',rot=206)    #path alone (contains pts_file)
if 2 in tests:v = vw.Viewer(cif_file='dat/glycine/alpha_glycine.cif')       #cif_file alone
if 3 in tests:v = vw.Viewer(pets='dat/pets/glycine.pts',init_opts='i',F='Ig',rot=206)              #pets alone
if 4 in tests:v = vw.Viewer(bloch='dat/bloch/Si110_200keV_bloch.pkl') #bloch alone

if 5 in tests:v = vw.Viewer(path='dat/Si',cif_file='Si',u=[0,0,1])                  #path and cif
if 6 in tests:v = vw.Viewer(path='dat/Si',bloch='dat/bloch/Si110_200keV_bloch.pkl') #path and bloch
if 7 in tests:v = vw.Viewer(path='dat/glycine',pets='dat/pets/glycine.pts')         #path and pets

#path and cif and bloch
if 8 in tests:v = vw.Viewer(path='dat/Si',cif_file='Si',bloch='dat/bloch/Si110_200keV_bloch.pkl')

#path and cif and pets
if 9 in tests:v = vw.Viewer(path='dat/glycine',cif_file='dat/glycine/alpha_glycine.cif',pets='dat/pets/glycine.pts')

#path and pets and bloch
if 10 in tests:v = vw.Viewer(path='dat/glycine',
    pets='dat/pets/glycine.pts',bloch='dat/glycine/alpha_glycine-103_200keV_bloch.pkl')

#path and cif_file pets and bloch
if 11 in tests:v = vw.Viewer(path='dat/glycine',cif_file='dat/glycine/alpha_glycine.cif',
    pets='dat/pets/glycine.pts',bloch='dat/glycine/0001_bloch.pkl')



from utils import* #.displayStandards import stddisp
import time
import multislice as mupy
from structure_factor import get_pendulossung
from postprocess import load_multi_obj as loadm

#########################################################################
#### def : runs
#########################################################################
def Si100_pendullosung_fit(filename='dat/Silicon/Si100/Si100_autoslic.pkl'):
    multi = loadm(filenames)
    hk,t,re,im,I = multi.get_beams(bOpt='f')
    I22 = I[1]/I[1].max()
    xi = get_pendulossung(name='Si',miller=[0,2,2],keV=200,opt='p'); #xi = 320
    Ikin = (pi*t/xi)**2
    Idyn = np.sin(pi*t/xi)**2
    plts=[[t,I22,'g','$I_{022}^{(autoslic)}$']]
    plts+=[[t,Ikin,'b--','$kin$'],[t,Idyn,'r--','$dyn-2$']]
    stddisp(plts,labs=['$thickness(\AA)$','$I$'],
        xylims=[0,350,0,1.2],lw=2,legLoc='upper right',axPos=1,setPos=1,
        opt=pOpt,name='docs_fig/Si100_I22.svg')#,title='$Si_{[100]}$')

def Si100_autoslic(pOpt='p',ppopt='',nz=70,**kwargs):
    name = 'dat/Silicon/Si100'
    multi = mupy.Multislice(name,mulslice=False,tail='%d' %nz,
        NxNy=512,repeat=[6,6,nz],slice_thick=1.375,Nhk=5,
        **kwargs)
    if ppopt:
        if not multi.check_simu_state() == 'done' : multi.p.wait()
        if 'b' in ppopt :
            multi.beam_vs_thickness(bOpt='f',opt=pOpt,setPos=1,axPos=1,
                name='docs_fig/Si100_Ihk.svg',cm='Greens')#,xylims=[])
        if 'p' in ppopt:
            name='docs_fig/Si100_autoslic_pattern_%dA.png' %(round(multi.thickness))
            multi.pattern(opt=pOpt,cmap='gray',imOpt='ch',
                name=name)
    return multi

def Si110_autoslic(opt='',fopt='w',pOpt=''):
    name = 'dat/Silicon/Si110'
    multi = mupy.Multislice(name,mulslice=False,
        slice_thick=1.375,NxNy=512,repeat=[10,10,120],Nhk=5,
        v=1,opt=opt,fopt=fopt)
    if pOpt:
        time.sleep(0.1)
        if not multi.check_simu_state(0) == 'done' : multi.p.wait()
        # multi.beam_vs_thickness(bOpt='f',opt=pOpt,setPos=1,axPos=1,
        #     name='docs_fig/Si110_Ihk.svg',cm='jet')#,xylims=[])
        multi.pattern(Iopt='Icnsl',cmap='gray',imOpt='ch',xylims=[0,12,0,12],
            name='docs_fig/Si110_autoslic_pattern.png',axPos='T',setPos=1,opt=pOpt)
    return multi

def Si110_mulslice(opt='',pOpt=''):
    name = 'dat/Silicon/Si110'
    multi = mupy.Multislice(name,mulslice=True,
        NxNy=512,repeat=[1,1,120],Nhk=5,
        v=1,opt=opt)
    if pOpt:
        multi.beam_vs_thickness(bOpt='f',opt=pOpt,setPos=1,axPos=1,
            name='docs_fig/Si110_Ihk_mulslice.svg',cm='jet')#,xylims=[])
    return multi


########################################################################
if __name__ == "__main__":
    plt.close('all')
    #multi = load_multi_obj('dat/Silicon/Si110/Si110_mulslice.pkl')
    #multi=Si100_autoslic(opt='r',ppopt='p',nz=60,fopt='f',v=1,pOpt='ps')
    #Si100_pendullosung_fit(filename='dat/Silicon/Si100/Si100_autoslic.pkl')
    #multi=Si110_mulslice(opt='r',pOpt='s')
    #multi=Si110_autoslic(opt='rs',pOpt='ps',fopt='')



from utils import*
plt.close('all')
path='../figures/'

t = np.linspace(1,1000,1000)
K = 1/0.025
Ug,Uh = 0.1,0.08
Sg,Sh = 0,0.02

Ugh = Ug-Uh
Ug_e = Ug - Uh*Ugh/(2*K*Sh)
So_e = - (abs(Uh)**2/(2*K*Sh))/(2*K)
Sg_e = (2*K*Sg - abs(Ugh)**2/(2*K*Sh))/(2*K)
Se = Sg_e-So_e
Ig = abs(Ug_e)**2/(K**2*Se**2+abs(Ug_e)**2)*np.sin(np.pi*t/K*np.sqrt(K**2*Se**2+abs(Ug_e)**2))**2

print(2*K*Sh,Ug)
print(K/Ug,K/Ug_e)
print(Sg_e)

plts = [[t,Ig,'b','']]
dsp.stddisp(plts,labs=['Thickness($A$)','$I_g$'],lw=2)



import importlib as imp
from utils import*                  ;imp.reload(dsp)
import multislice.multi_2D as MS2D  ;imp.reload(MS2D)
import scipy.fftpack as fft
plt.close('all')
path='../../multislice/docs_fig/multi2D/single_array/'



Ai = np.array([0.1,0.25,0.26,0.27,1.5])
def Va( r,Za):
    return np.exp(-(r/Ai[Za])**2)
def Vq(r,Za) :
    fv = np.zeros(r.shape)
    fv[np.abs(r)<Ai[Za]] = 0.5
    return fv

def show_potential_profile(Za=3):
    r = np.linspace(0,1,1000)
    plts = [[r,Va(r,Za),'b','atom'],[r,Vq(r,Za),'r','qdot']]
    dsp.stddisp(plts,labs=['$r$','$V(r)$'],lw=2,fonts='2',
        name=path+'potential.eps',opt='ps')

class SingleArrays:
    def __init__(self,keV=100,nx=2**14,nz=1000,ax=2000,Za=3,
            bzs=[2,5],eps=1,Vatom=True,plot_opt=False):
        self.keV=keV
        self.ax=ax
        self.bzs=np.array(bzs)
        self.eps=eps

        bz0 = bzs[0]
        self.x0,self.z0 = np.linspace(0,ax,nx),np.linspace(0,bz0,nz)
        self.xa,self.za,self.Za = ax/2,bz0/2,Za
        self.x,self.z = np.meshgrid(self.x0,self.z0)
        self.r      = np.sqrt((self.x-self.xa)**2+(self.z-self.za)**2)
        self.Vatom  = Vatom
        self.V      = [Vq,Va][Vatom]

        print('..computing fv...')
        self.fv = self.V(self.r,self.Za)
        if plot_opt:self.plot_v()

    def run(self,Nz=100,iZv=None):
        if not iZv:iZv = Nz
        mp0 = np.zeros(self.bzs.shape,dtype=object)
        pattern = [self.x0,self.z0,self.fv]
        for i,bz in enumerate(self.bzs):
            mp0[i] = MS2D.Multi2D(pattern,self.ax,bz,self.keV,
                    Nx=1,dz=bz,nz=Nz,#nz=int(Nz/bz),
                    ppopt='',iZs=1,iZv=iZv,eps=self.eps,copt=0)
        self.mp0 = mp0

    def plot_v(self):
        return MS2D.plot_v(self.x,self.z,self.fv,self.ax,self.bz0,self.xa,self.za)

    def Qz_show(self,iBz=slice(0,None),iZs=1,opts='N',name='',**kwargs):
        for i,bz in enumerate(self.bzs[iBz]):
            self.mp0[i].Qz_show(iZs=iZs,lw=2,opts=opts,title='bz=%d' %bz,
                name=path+name+'bz%d.svg' %i,**kwargs)

    def plot_bzs(self,idx=-1,**kwargs):
        bzs,mp0 = self.bzs,self.mp0

        cs = dsp.getCs('Spectral',bzs.size)
        for i,bz in enumerate(self.bzs): self.mp0[i].psi_qz[idx,0]=0
        I = [fft.fftshift(mp0[i].psi_qz[idx,:]) for i,bz in enumerate(bzs)]
        plts = [[fft.fftshift(mp0[i].q),I[i]/max(I[i]),cs[i],'bz=%d'%bz] for i,bz in enumerate(bzs)]
        tle = '''diffraction pattern with varying interlattice at
            Nz=%d space eps=%.2f, %s''' %(idx,self.eps,['Vqdot','Vatom'][self.Vatom])
        fig,ax = dsp.stddisp(plts,title=tle,**kwargs)



def show_proj_potential(s0,s1,**kwargs):
    plts=[[s0.mp0[0].x,s0.mp0[0].Vz[0,:],'b--','$\epsilon=%.2f$' %s0.eps],
          [s1.mp0[0].x,s1.mp0[0].Vz[0,:],'r','$\epsilon=%.2f$' %s1.eps],
          ]
    dsp.stddisp(plts,lw=2,labs=[r'$x(\AA)$','$V_z$'],
        name=path+'projV.eps',**kwargs)

def show_kd(s0,s1,idx,xylims=[],**kwargs):
    for i,bz in enumerate(s0.bzs): s0.mp0[i].psi_qz[idx,0]=0
    for i,bz in enumerate(s1.bzs): s1.mp0[i].psi_qz[idx,0]=0
    qa = np.linspace(0,4,300)
    qa_x,qa_y = np.meshgrid(qa,qa)
    qas = np.sqrt(qa_x**2+qa_y**2)
    t = np.linspace(0,np.pi,1000)
    st,ct,ka = np.sin(t),np.cos(t),40
    plts0 = [[ka*st,ka*(1-ct),'r','ewald']]
    for i,bz in enumerate(s0.bzs):
        I0 = s0.mp0[i].psi_qz[idx,:]
        I1 = s1.mp0[i].psi_qz[idx,:]
        plts=[[s0.mp0[i].q,I0/I0.max(),'b--','$\epsilon=%.2f$' %s0.eps],
              [s1.mp0[i].q,I1/I1.max(),'r'  ,'$\epsilon=%.2f$' %s1.eps],
              ]

        f = np.exp(-(np.pi*qas*Ai[s0.Za])**2)*np.cos(qa_x*bz/(4*np.pi))
        dsp.stddisp(plts0,im=[qa_x,qa_y,np.abs(f)],lw=2,labs=['$q_z$','$q_x$'],
            xylims=[0,xylims[1],0,xylims[1]],cmap='viridis',title='bz=%d' %bz,
            name=path+'kd%d_V.png' %(i+1),**kwargs)
        dsp.stddisp(plts,lw=2,labs=[r'$q(\AA^{-1})$','$I$'],
            name=path+'kd%d_D.eps' %(i+1),xylims=xylims,**kwargs)

# show_kd(s0,s1,1,xylims=[0,8,0,0.02],fonts='3',opt='p',caxis=[0,0.001])

# show_potential_profile(Za=3)


ax,nx = 1000,2**15
xy = ax/2 +2*np.array([-1,1])
bzs = np.array([8,16])#[[0]]

s0 = SingleArrays(bzs=bzs,eps=0.001,Vatom=True,plot_opt=False,nx=nx,ax=ax)
s0.run(Nz=101)
# s0.mp0[0].Tz_show(iSz=slice(0,None,1),Vopt='VT',lw=2,xylims=['x',xy[0],xy[1]])
s0.Qz_show(iBz=[0,-1],iZs=np.arange(0,4,1),xylims=[0,8,0,0.00001],opts='',opt='p')
# s0.plot_bzs(idx=0,lw=2,xylims=[-4,4,0,1],opt='p')
# s0.plot_bzs(idx=-1,lw=2,xylims=[-4,4,0,1],opt='p')


s1 = SingleArrays(bzs=bzs,eps=0.01,Vatom=True,plot_opt=False,nx=nx,ax=ax)
s1.run(Nz=101)
# s1.mp0[0].Tz_show(iSz=slice(0,None,1),Vopt='VT',lw=2,xylims=['x',xy[0],xy[1]])
# s1.Qz_show(iBz=[0,-1],iZs=np.arange(0,50,15),xylims=['x',-4,4],opts='N',opt='p')
# s1.plot_bzs(idx=0,lw=2,xylims=[-4,4,0,1],opt='p')
# s1.plot_bzs(idx=-1,lw=2,xylims=[-4,4,0,1],opt='p')
#
# s0.plot_bzs(idx=1,lw=2,xylims=[-4,4,0,1],opt='p')
# s1.plot_bzs(idx=1,lw=2,xylims=[-4,4,0,1],opt='p')



# show_proj_potential(s0,s1,xylims=[xy[0],xy[1],0,0.3],fonts='2',opt='ps')



from utils import*
from scipy.integrate import trapz
plt.close('all')
path='../figures/'

Ikf = lambda t,sg,xi_g:(np.pi*t/xi_g)**2*np.sinc(t*sg)**2
Idf = lambda t,sg,xi_g:(np.pi*t/xi_g)**2*np.sinc(t/xi_g*np.sqrt(1+(sg*xi_g)**2))**2
def integrate_th_full(Ts=np.linspace(1,5000,5000),xi=500,iTs=0,**kwargs):
    '''Integrate the full rocking curve for each thickness
    - Ts : thicknesse
    - xi : Pendullosung
    - iTs : selected indices in Ts for displaying rocking curves
    '''
    nt = 50
    Ikin,Idyn = np.zeros(Ts.shape),np.zeros(Ts.shape)
    for iT,t in enumerate(Ts):
        sg = np.linspace(-1,1,20000)*nt/t
        Ikin[iT] = trapz(Ikf(t,sg,xi),sg)
        Idyn[iT] = trapz(Idf(t,sg,xi),sg)
    plts = [[Ts,Ikin,'b-','$kin$'],[Ts,Idyn,'r-','$dyn$']]
    dsp.stddisp(plts,labs=[r'$z(\AA)$','Integrated $I$'],**kwargs)

    #plot details
    if isinstance(iTs,int):
        if iTs :
            iTs=slice(0,None,iTs)
        else:
            iTs = slice(0,0)
    pltsD, cs = [], dsp.getCs('jet',Ts[iTs].size)
    for i,t in enumerate(Ts[iTs]):
        sg = np.linspace(-1,1,2000)*nt/t
        Ik = Ikf(t,sg,xi)
        Id = Idf(t,sg,xi)

        pltsD += [[sg,Id,[cs[i],'-'],r'$z=%.1f\AA$' %t]]

        plts = [[sg,Id,'r-','$dyn$']]
        # plts += [[sg,Ik,'b-o','$kin$']]
        dsp.stddisp(plts,labs=[r'$s_g$','$I$'],lw=2,title=r'$z=%f\AA$' %t)
    # dsp.stddisp(pltsD,labs=[r'$s_g$','$I$'],lw=2)#,xylims=[-1,1,0,1])

def plot_rocking_curves(Ts=np.linspace(1,5000,10),xi=500,**kwargs):
    pltsD,cs = [], dsp.getCs('viridis',Ts.size)
    for i,t in enumerate(Ts):
        sg = np.linspace(-0.005,0.005,1000)
        pltsD += [[sg,Idf(t,sg,xi),[cs[i],'-'],r'$z=%.1f\AA$' %t]]
    dsp.stddisp(pltsD,labs=[r'$s_g$','$I$'],**kwargs)


integrate_th_full(Ts=np.linspace(1,3000,100),xi=500,iTs=0,
    xylims=[0,5000,0,0.005],lw=2,name=path+'Iint_kin_dyn.svg',opt='p')
# plot_rocking_curves(Ts=np.array([0.75,1,1.25,1.5])*500,xi=500,
#     xylims=[-0.005,0.005,0,1],lw=2,name=path+'Iint_kin_dynR.svg',opt='ps')



'''
Defines 2D lattices with some utilities
'''
from utils import*
from utils import displayStandards as dsp
import numpy as np

pp_types = ['p1','pg','pm','cm', 'p2','pgg','pmm','cmm','pmg',
            'p4','p4m','p4g','p3','p3m1','p31m','p6','p6m']; #print(pp_types)
pp_lattices = ['oblique']+['rect']*3+['oblique']+['rect']*4+['square']*3+['hex']*5
wallpp_lat = dict(zip(pp_types,pp_lattices))

__all__ = ['get_lattice_vec','get_miller','reciprocal_lattice_2D',
    'plot_lattice']

##### 2D
def get_lattice_vec(lat_type='',wallpp_type=None,a=1,b=1,alpha=90,v=0) :
    ''' Computes lattice vectors from :
        - a,b,angle : lattice parameters
        - lat_type  : lattice type  [hex,square,rect,oblique]'''
    pi = np.pi
    if wallpp_type : lat_type = wallpp_lat[wallpp_type]
    if lat_type == 'hex' :
        lattice_vec = np.array([[1,0],[cos(pi/3),sin(pi/3)]])*a
    elif lat_type == 'square' :
        lattice_vec = np.array([[1,0],[0,1]])*a
    elif lat_type == 'rect' :
        lattice_vec = np.array([[a,0],[0,b]])
    elif lat_type == 'oblique' :
        angle=pi/180*alpha
        lattice_vec = np.array([[a,0],[b*cos(angle),b*sin(angle)]])
        if v : lattice_vec = np.array([[a*cos(angle),a*sin(angle)],[0,b]])
    else :
        printf(red,'not a valid lattice type. choose : ''', ['hex','square','rect','oblique'],
               'or select a wallpaper type : ',pp_types ,black)
        return None
    return lattice_vec

# misc
def get_miller(lattice_vec,nh,nk,ax=None):
    ''' get the miller indices and correspoding vectors in format :
            nh.nk x 2 '''
    n_h,n_k = np.arange(nh),np.arange(nk)
    miller = np.array(np.meshgrid(n_h,n_k)).T.reshape(((nh)*(nk),2))
    u_h = lattice_vec.T.dot(miller.T).T
    if ax :
        for u in u_h:
            ax.plot([0,u[0]],[0,u[1]],'s-g')
    return u_h,miller

def reciprocal_lattice_2D(a1,a2):
    Rot90 = np.array([[0,1],[-1,0]])
    b1 = Rot90.dot(a2)/(a1.dot(Rot90.dot(a2)))#*2*np.pi
    b2 = Rot90.dot(a1)/(a2.dot(Rot90.dot(a1)))#*2*np.pi
    return b1,b2


# Display
def plot_lattice(lattice_vec,nh=3,nk=3,hOpt=True,**kwargs) :
    '''Plot lattice grid :
    - nh,nk : number of cells along each direction
    - hOpt : additional lines for hexagonal
    '''
    gridcolor=(0.75,0.75,0.75)
    a,b=lattice_vec
    n_h,n_k = np.arange(nh+1),np.arange(nk+1)
    line_h = [h*a + np.array([[0,0],nk*b]) for h in range(nh+1)]
    line_k = [k*b + np.array([[0,0],nh*a]) for k in range(nk+1)]
    plts = [[xy[:,0],xy[:,1],gridcolor,''] for xy in line_h+line_k]
    if hOpt :
        P = (a+b)
        n,m,nn,nm = a,b,nh,nk
        if nk>nh : n,m,nn,nm = b,a,nk,nh
        line_hex  = [h*n + np.array([[0,0],(nn-h)*P]) for h in range(nm,nn)]
        line_hex += [k*m + np.array([[0,0],(nm-k)*P]) for k in range(nm)]
        if not nk==nh :
            line_hex += [h*n + np.array([[0,0],nm*P]) for h in range(nn-nm)]
        plts += [[xy[:,0],xy[:,1],[gridcolor,'--'],''] for xy in line_hex]
    fig,ax=dsp.stddisp(plts,**kwargs)
                   # xylims=xylims,mg=0,changeXYlims='x' in pOpt,
                   # gridOn='g' in pOpt,ticksOn='t' in pOpt,equal='e' in pOpt,
                   # setPos='s' in pOpt,)
    return fig,ax

##########################################################################
#### test
##########################################################################
def _test_plot_all() :
    for lat in ['hex','square','rect','oblique'] :
        plot_lattice(get_lattice_vec(lat,a=1,b=3,alpha=50),nh=6,nk=3,pOpt='et')
def _test_plot_hex() :
    vec = get_lattice_vec('hex',a=1)
    plot_lattice(vec,nh=6,nk=6,xylims=[2,5,0,3],pOpt='xehp')
def _test_miller():
    vec = get_lattice_vec('oblique',a=1,b=1.5,alpha=20)
    nh,nk=1,0#3,4
    fig,ax=plot_lattice(vec,nh,nk,xylims=[0,2,0,3],pOpt='t')
    u_h,miller = get_miller(vec,nh,nk,ax);print(u_h,miller)

if __name__=='__main__' :
    #_test_plot_all()
    #_test_plot_hex()
    #_test_miller()
    print(green+'success'+black)
